<?php

class IniFile extends BaseObject {

 var $d_filename;
 var $d_parent;
 var $d_wildcarts;

 var $d_sectionNames;
 var $d_sectionValues;
 var $d_sectionEntries;
 var $d_fetched;

 function IniFile($filename) {
 $this->d_filename = $filename;
 $this->d_parent = "parent";
 $this->d_wildcharts = true;
 $this->d_fetched = false;
 $this->d_sectionNames = array();
 $this->d_sectionValues = array();
 $this->d_sectionEntries = array();
 }

 function setUseWildcarts($value) {
 $this->d_wildcarts = $value;
 }

 function setParentSection($value) {
 $this->d_parent = $value;
 }

 function getSection($section) {
 $s = $this->_loadSection($section);
 if ($s === false) return false;
 return $this->_getSection($s);
 }
 
 




 function getSections() {
 if (!$this->d_fetched) $this->_fetchIniFile();
 return $this->d_sectionNames;
 }

 function getSectionValues() {
 if (!$this->d_fetched) $this->_fetchIniFile();
 return $this->d_sectionValues;
 }

 function _fetchIniFile() {
 $this->_fetchSection(false);
 $this->d_fetched = true;
 }

 function _fetchSection($section) {
 $f = fopen($this->d_filename, "r");
 
 $lsection = "";
 $reading = false;
 while (!feof($f)) {
 $s = fgets($f, 1024);
 if (ereg("^ *\[(.*)\]",$s, $regs)) {
 if ($reading && $section !== false) return $lsection;
 
 if ($section === false || $this->_matchSection($section, $regs[1])) {
 $reading = true;
 $lsection = strtolower($regs[1]);
 $this->d_sectionNames[$lsection] = $regs[1];
 $this->d_sectionValues[$lsection] = array();
 $this->d_sectionEntries[$lsection] = array();
 }
 } else if ($reading && ereg("^ *([^= ]+) *= *([^ ].*).*$", $s, $regs)) {
 $key = strtolower($regs[1]);
 $this->d_sectionEntries[$lsection][$key] = $regs[1];
 $val = str_replace( array("\n","\r"), "", $regs[2] );
 $this->d_sectionValues[$lsection][$key] = $val; 
 }
 }
 
 fclose($f);
 return false;
 }

 function _findSection($section) {
 foreach ($this->d_sectionNames as $s=>$val) {
 if ($this->_matchSection($section, $s)) return $s;
 }
 return false;
 }

 function _loadSection($section) {
 if (($s = $this->_findSection($section)) === false) {
 $s = $this->_fetchSection($section);
 }
 return $s;
 }

 function _getSection($section, $visited = array()) {
 $rs = $this->d_sectionValues[$section];

 if ($this->d_parent && ($parent = $rs[strtolower($this->d_parent)]) !== NULL) {
 $visited[$section] = true;
 if (($p = $this->_loadSection($parent)) !== false) {
 if (!$visited[$p]) $rs += $this->_getSection($p, $visited);
 }
 }
 
 return $rs;
 }

 function _matchSection($search, $iniSection) {
 if (!$this->d_wildcards) return (strtolower($search) == strtolower($iniSection));

 $s = $iniSection;
 $s = str_replace("\\","\\\\",$s);
 $s = str_replace("(","\\(",$s);
 $s = str_replace(")","\\)",$s);
 $s = str_replace("[","\\[",$s);
 $s = str_replace("]","\\]",$s);
 $s = str_replace(".","\\.",$s);
 $s = str_replace("_","\\_",$s);
 $s = str_replace("*","(.*)",$s);
 
 return eregi($s, $search);
 }
}

?>
